#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "helper.h"

#define SIZE (4*1024*1024)
#define TIMES 40

int nb_threads = 1;
size_t length = SIZE;
int repetitions = TIMES;
int verbose = 0;

int helper_parse_arguments(int argc, char* argv[] )
{
    int i;

    if( argc > 1 ) {
        for( i = 1; i < argc; i++ ) {
            if( '-' == argv[i][0] ) {
                if( 't' == argv[i][1] ) {
                    nb_threads = atoi(argv[i+1]);
                    if( nb_threads <= 0 ) {
                        printf("Number of threads must be > 0\n");
                        return -1;
                    }
                    i++;  /* jump one arg */
                    continue;
                }
                if( 'n' == argv[i][1] ) {
                    repetitions = atoi(argv[i+1]);
                    if( repetitions <= 0 ) {
                        printf("Number of repetitions must be > 0\n");
                        return -1;
                    }
                    i++;  /* jump one arg */
                    continue;
                }
                if( 'v' == argv[i][1] ) {
                    verbose = 1;
                    continue;
                }
                printf("Unknown argument %s\n", argv[i]);
            }
            length = atoi(argv[i]);
            break;
        }
        if( length <= 0 ) {
            printf("The supplied value must be positive\n");
            return -1;
        }
    }
    return 0;
}

int helper_print_time(long* array)
{
    int i;

    printf("%ld;%d;%g;%g;", length, nb_threads,
           compute_mean(array, repetitions),
           compute_std(array, repetitions));
    if( verbose ) {
        for( i = 0; i < repetitions; i++ )
            printf("%g;", (double)array[i]);
    }
    printf("\n");
    return 0;
}

void TYPE_NAME(helper_init_array_)(TYPE* array, size_t length)
{
    int i;
    
    for( i = 0; i < length; i++ ) {
        array[i] = (TYPE)arc4random_uniform(length) / (TYPE)length;
    }
}

TYPE* TYPE_NAME(helper_build_array_)(size_t length)
{
    int i;
    TYPE* array = (float*)malloc(length * sizeof(TYPE));
    
    for( i = 0; i < length; i++ ) {
        array[i] = (TYPE)arc4random_uniform(length) / (TYPE)length;
    }
    return array;
}

double compute_mean(long* array, size_t length)
{
    int i;
    double result = 0.0;

    for( i = 0; i < length; i++ ) {
        result += (double)(array[i]);
    }
    return result / (double)length;
}

double compute_std(long* array, size_t length)
{
    int i;
    double variance = 0.0;
    double mean = compute_mean(array, length);

    for( i = 0; i < length; i++ ) {
        variance += pow(((double)array[i] - mean), 2.0);
    }
    return sqrt(variance / length);
}
