#include <stdlib.h>
#include <sys/time.h>
#include <stdio.h>
#include <omp.h>
#include <math.h>

#include "helper.h"

// gcc -Wall min_v2.c helper.c -o min_v2 -fopenmp -std=c11

TYPE compute_min(TYPE* array, size_t length)
{
    TYPE selected = array[0];
#pragma omp parallel num_threads(nb_threads) shared(selected)
    {
        TYPE lselected = array[0];
#pragma omp for
        for( int i = 1; i < length; i++ ) {
            if( lselected > array[i] )
                lselected = array[i];
        }
#pragma omp critical (selection)
        {
            if(selected > lselected)
                selected = lselected;
        }
    }
    return selected;
}

int main( int argc, char* argv[] )
{
    TYPE *array, selected;
    int i;
    long *elapsed_time;
    struct timeval start, end;

    helper_parse_arguments(argc, argv);

    array = TYPE_NAME(helper_build_array_)(length);
    elapsed_time = (long*)malloc(repetitions * sizeof(long));
    
    // Run it once to start up the engine
    selected = compute_min( array, length ); (void)selected;

    for( i = 0; i < repetitions; i++ ) {
        gettimeofday(&start, NULL);

        selected = compute_min( array, length );

        gettimeofday(&end, NULL);
        elapsed_time[i] = ((end.tv_sec - start.tv_sec) * 1000000 +
                           (end.tv_usec - start.tv_usec)), omp_get_num_threads();
    }
    printf("# Found the minimum value out of %ld elements in %g/%g usec using %d threads\n",
           length, compute_mean(elapsed_time, repetitions),
           compute_std(elapsed_time, repetitions), nb_threads);

    helper_print_time(elapsed_time);
    
    free(array);
    free(elapsed_time);
    return 0;
}
